/*
*    Language Investigator and Stractural Analyzer (LISA) -
*    A descriptive programming language
*
* LICENCE :             LGPL
* FILE:			abstractsyntaxtree.h
*
* PURPOSE:		Abstract Syntax Tree
*
* CREATED AT:		December 16, 2011
*
* AUTHOR: 		Seyed Pooria Madani Kochak (s.p.madani@gmail.com)
*
* Project Manager:	Dr. Cezar Campeanu (ccampeanu@upei.ca)
*/

/*
* This file holds declaration of the abstract syntax tree for LISA.
* The syntax tree is created using BISON/YACC impilictly. However,
* the nodes of the syntax tree is defined in this file.
*
* Notes:
* (1) Each of the enums contains a char pointer called "code".
* "code" will hold intermediate code generated so far till that node,
* and will get pushed up (and might get modified).
*
* (2) Each node contains "type". This will be used in type checking
* and the type also get pushed up to upper nodes and might get changed
* in between.
*
* (3) Above each struct declaration, YACC grammar production is given
* inorder to be able to match the content of the struct.
*
* Revision History
* --------------------------------------------------------------------------
* Seyed Pooria Madani - Creation							December 16, 2011
*/


#ifndef	AST_H
#define AST_H
#include "symboltable.h"

/*
expression:
	variable exprop expression
	| simpleexpression
	;
*/
struct Expression {
    enum symbol_node_type		type;
    struct 	Simple_Expression 	*sm_expression;
    int							operator_index;
    int 						variable_index;
    struct Expression		  	*expression;
    unsigned char				*code;
};

/*
simpleexpression:
	simpleexpression OR orexpression
	| orexpression
	;
*/
struct Simple_Expression {
    enum symbol_node_type type;
    struct 	Simple_Expression 	*sm_expression;
    struct 	Or_Expression	  	*or_expression;
    int							index_operator;
    unsigned char				*code;

};

/*
orexpression:
	orexpression AND unaryrelationexpression
	| unaryrelationexpression
	;
*/
struct Or_Expression {
    enum symbol_node_type type;
    struct Or_Expression	  			*or_expression;
    struct Unary_Relation_Expression	*unary_relation_expression;
    int									index_operator;
    unsigned char						*code;
};

/*
unaryrelationexpression:
	'!' unaryrelationexpression
	| relationexpression
	;
*/
struct Unary_Relation_Expression {
    enum symbol_node_type type;
    struct Unary_Relation_Expression	*unary_relation_expression;
    struct Relation_Expression			*relation_expression;
    unsigned char						*code;

} ;


/*
relationexpression:
	addexpression relop addexpression
	| addexpression
	;
*/
struct Relation_Expression {
    enum symbol_node_type type;
    struct Add_Expression 		*add_expression1;
    struct Add_Expression		*add_expression2;
    int 						index_relation_operator; //Index to symbol table
    unsigned char				*code;

} ;

/*
  addexpression:
   	addexpression addop term
   	| term
   ;
*/
struct Add_Expression {
    enum symbol_node_type type;
    struct Add_Expression 			*add_expression;
    struct Term						*term;
    int								index_add_operator;//Index to symbol table
    unsigned char					*code;
} ;


/*
	term:
		term multop factor
		| factor
	;

*/
struct Term {
    enum symbol_node_type type;
    struct Term			*term;
    struct Factor		*factor;
    int					index_multiplication_operator; //Index to symbol table
    unsigned char		*code;
} ;


/*
factor:
	'(' simpleexpression ')'
	| constant
	;
*/
struct Factor {
    enum symbol_node_type type;
    struct Simple_Expression	*simple_expression;
    struct Constant				*constant;
    unsigned char				*code;

} ;

/*
constant:
	INTEGER	| TRUE	| FALSE	| NEXT	| HASNEXT	| STRINGLITERAL
	| variable
	| function
	;
	 *
*/
struct Constant {
    enum symbol_node_type 		type;
    int 						sindex_constant; //Index to symbol table
    struct Function				*function;
	struct Variable				*variable;
    unsigned char				*code;
};



/*
variable:
	ID
	| ID '[' INTEGER ']
	;
*/
struct Variable {
    enum symbol_node_type 		type;
    int 						id_index; //Index to symbol table;
	int							array_index; //Current addressed index
    unsigned char				*code;
};

/*
function:
	ID '(' parameterlist ')'
*/
struct Function {
    enum symbol_node_type 			type;
    int 							id_index; //Index to symbol table;
    struct Parameter_List			*parameterlist; //Pointer to array of Parameters
    unsigned char					*code;
} ;

/*
parameterlist:
	simpleexpression
	| type
	|parameterlist ',' simpleexpression
	| NULL
	;
*/
struct Parameter_List {
    enum symbol_node_type 			type;
    struct Simple_Expression 		*simple_expression;
    struct Parameter_List			*parameterlist; //Pointer to array of Parameters
    unsigned char    				*code;
};



#endif
