/*
*    Language Investigator and Stractural Analyzer (LISA) - 
*    A descriptive programming language

* LICENCE :             LGPL 
*
*
* FILE:					functions.c
* 
* PURPOSE:				Predefined functions implementation
*
* CREATED AT:			December 15, 2011
*
* AUTHOR: 		Seyed Pooria Madani Kochak (s.p.madani@gmail.com)
* 
* Project Manager:	Dr. Cezar Campeanu (ccampeanu@upei.ca)
*/

/*
* This file holds implementation of functions declaraed in functions.h
* 
* Revision History
* --------------------------------------------------------------------------
* Seyed Pooria Madani - Creation							December 15, 2011
*/


#include "functions.h"
#include "icg.h"
#include <stdlib.h>
#include <stdio.h>

/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to retrive the size of the parameter
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *size(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char *functionName; //Hold the function name should be called
    /*
     * (1) Size will only accept once parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'size' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        functionName = "NFA::size\0";
        break;
    case NODETYPE_DFA:
        functionName = "DFA::size\0";
        break;
    case NODETYPE_STRING:
        functionName = "STRING::size\0";
        break;
    case NODETYPE_REGEX:
        functionName = "REGEX::size\0";
        break;
    default:
        yyerror("The parameter for 'size' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size); //Always set memory to NULL


	/*
	 * The pattern to be created:
	 * <functionName> ( <var_node.value_name> )
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    generated_code[strlen(functionName)] = '(' ;
    memcpy(&(generated_code[strlen(functionName)+1]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;    

}


/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to retrive the size of the parameter
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *max(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char *functionName; //Hold the function name should be called
    /*
     * (1) Size will only accept once parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'max' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        functionName = "NFA::maxstring\0";
        break;
    case NODETYPE_DFA:
        functionName = "DFA::maxstring\0";
        break; 
	default:
        yyerror("The parameter for 'max' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size); //Always set memory to NULL


	/*
	 * The pattern to be created:
	 * <functionName> ( <var_node.value_name> )
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    generated_code[strlen(functionName)] = '(' ;
    memcpy(&(generated_code[strlen(functionName)+1]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;    

}





/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to retrive the size of the parameter
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *isfinite(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char *functionName; //Hold the function name should be called
    /*
     * (1) isfinite will only accept once parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'isfinite' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        functionName = "NFA::isFinite\0";
        break;
    case NODETYPE_DFA:
        functionName = "DFA::isFinite\0"; //Same function will get called
        break;
    default:
        yyerror("The parameter for 'isfinite' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size); //Always set memory to NULL


	/*
	 * The pattern to be created:
	 * <functionName> ( <var_node.value_name> )
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    generated_code[strlen(functionName)] = '(' ;
    memcpy(&(generated_code[strlen(functionName)+1]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;    

}



/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to retrive the size of the parameter
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *iscomplete(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char *functionName; //Hold the function name should be called
    /*
     * (1) isfinite will only accept once parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'iscomplete' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        functionName = "NFA::isComplete\0";
        break;
    case NODETYPE_DFA:
        functionName = "DFA::isComplete\0"; //Same function will get called
        break;
    default:
        yyerror("The parameter for 'iscompelete' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size); //Always set memory to NULL


	/*
	 * The pattern to be created:
	 * <functionName> ( <var_node.value_name> )
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    generated_code[strlen(functionName)] = '(' ;
    memcpy(&(generated_code[strlen(functionName)+1]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;    

}



/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to retrive the size of the parameter
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *isuniversal(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char *functionName; //Hold the function name should be called
    /*
     * (1) isuniversal will only accept once parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'isuniversal' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        functionName = "NFA::isUniversal\0";
        break;
    case NODETYPE_DFA:
        functionName = "DFA::isUniversal\0"; //Same function will get called
        break;
    default:
        yyerror("The parameter for 'isuniversal' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size); //Always set memory to NULL


	/*
	 * The pattern to be created:
	 * <functionName> ( <var_node.value_name> )
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    generated_code[strlen(functionName)] = '(' ;
    memcpy(&(generated_code[strlen(functionName)+1]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;    

}


/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to retrive the size of the parameter
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *isdeterministic(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char *functionName; //Hold the function name should be called
    /*
     * (1) isuniversal will only accept once parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'isdeterministic' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        functionName = "NFA::isDeterministic\0";
        break;
    case NODETYPE_DFA:
        functionName = "DFA::isDeterministic\0"; //Same function will get called
        break;
    default:
        yyerror("The parameter for 'isdeterministic' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size); //Always set memory to NULL


	/*
	 * The pattern to be created:
	 * <functionName> ( <var_node.value_name> )
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    generated_code[strlen(functionName)] = '(' ;
    memcpy(&(generated_code[strlen(functionName)+1]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;    

}


/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to reverse the parameter (one).
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *reverse(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) reverse will only accept one parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'reverse' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */


    unsigned char *hname = getHeapObjectName();


    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
    	sprintf(functionName,"*(%s= NFA::reverse(\0",hname);
        //functionName = "*hname = NFA::reverse(\0";
        break;
    case NODETYPE_DFA:
    	sprintf(functionName,"*(%s= DFA::reverse(\0",hname);
        //functionName = "*(hname = DFA::reverse(\0";
        break;
    case NODETYPE_REGEX:
    	sprintf(functionName,"*(%s= REs::reverse(\0",hname);
        //functionName = "*(REGEX::reverse(\0";
        break;
    default:
        yyerror("The parameter for 'reverse' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

	
	
    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);
	
	/*
	 * The pattern to be created:
	 * *(<functionName> ( <var_node.value_name> ))
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;

}


/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			for union of two parameters of same type.
* 			Only for types that union is applicable(ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *union_(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    //unsigned char *functionName; //Hold the function name should be called
    unsigned char functionName[100];
    /*
     * (1) union_ will only accept two parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist == NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'union_' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */
	 
    //Make sure both parameters are same type
    if (parameterlist->simple_expression->type != parameterlist->parameterlist->type)
        yyerror("Both parameters in 'union' has to be same type.");



    unsigned char *hname = getHeapObjectName();

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        sprintf(functionName,"*(%s= NFA::unionnfa(\0",hname);
    	//functionName = "*(hname = NFA::unionnfa(\0";
        break;
    case NODETYPE_DFA:
        //functionName = "*(hname = DFA::uniondfa(\0";
    	sprintf(functionName,"*(%s= DFA::uniondfa(\0",hname);
        break;
    case NODETYPE_REGEX:
        //functionName = "*(REGEX::unionregex(\0";
    	sprintf(functionName,"*(REGEX::unionregex(\0");
        break;
    default:
        yyerror("The parameter for 'union_' function is invalid.");
        break;
    }


    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */


    /*
	 * Is the union of both parameter.
	 * We have already verified the parameters. And we know
	 * the intermediate code for parameters are already generated.
	 * Thus, we only print that generated code inside ( and )
	 */
    code_size = strlen(functionName) + strlen(parameterlist->code) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),parameterlist->code,strlen(parameterlist->code));
    generated_code[strlen(functionName) + strlen(parameterlist->code)] = ')' ;
    generated_code[strlen(functionName) + strlen(parameterlist->code) + 1] = ')' ;

    return generated_code;   

}




unsigned char *concat(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    //unsigned char *functionName; //Hold the function name should be called
    unsigned char functionName[100];
    /*
     * (1) union_ will only accept two parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist == NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'concat' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    //Make sure both parameters are same type
    if (parameterlist->simple_expression->type != parameterlist->parameterlist->type)
        yyerror("Both parameters in 'concat' has to be same type.");



    unsigned char *hname = getHeapObjectName();

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        sprintf(functionName,"*(%s= NFA::concat(\0",hname);

        break;
    case NODETYPE_DFA:

    	sprintf(functionName,"*(%s= DFA::concat(\0",hname);
        break;
    case NODETYPE_REGEX:

    	sprintf(functionName,"*(REGEX::concat(\0");
        break;
    default:
        yyerror("The parameter for 'union_' function is invalid.");
        break;
    }


    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */


    /*
	 * Is the union of both parameter.
	 * We have already verified the parameters. And we know
	 * the intermediate code for parameters are already generated.
	 * Thus, we only print that generated code inside ( and )
	 */
    code_size = strlen(functionName) + strlen(parameterlist->code) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),parameterlist->code,strlen(parameterlist->code));
    generated_code[strlen(functionName) + strlen(parameterlist->code)] = ')' ;
    generated_code[strlen(functionName) + strlen(parameterlist->code) + 1] = ')' ;

    return generated_code;

}
unsigned char *shuffle(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    //unsigned char *functionName; //Hold the function name should be called
    unsigned char functionName[100];


    if (parameterlist->parameterlist == NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'shuffle' function");
    }


    //Make sure both parameters are same type
    if (parameterlist->simple_expression->type != parameterlist->parameterlist->type)
        yyerror("Both parameters in 'shuffle' has to be same type.");



    unsigned char *hname = getHeapObjectName();

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        sprintf(functionName,"*(%s= NFA::shuffle(\0",hname);
        break;
    case NODETYPE_DFA:
    	sprintf(functionName,"*(%s= DFA::shuffle(\0",hname);
        break;
    default:
        yyerror("The parameter for 'shuffle' function is invalid.");
        break;
    }


    code_size = strlen(functionName) + strlen(parameterlist->code) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),parameterlist->code,strlen(parameterlist->code));
    generated_code[strlen(functionName) + strlen(parameterlist->code)] = ')' ;
    generated_code[strlen(functionName) + strlen(parameterlist->code) + 1] = ')' ;

    return generated_code;

}


unsigned char *cross(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    //unsigned char *functionName; //Hold the function name should be called
    unsigned char functionName[100];


    if (parameterlist->parameterlist == NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'cross' function");
    }


    //Make sure both parameters are same type
    if (parameterlist->simple_expression->type != parameterlist->parameterlist->type)
        yyerror("Both parameters in 'cross' has to be same type.");



    unsigned char *hname = getHeapObjectName();

    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        sprintf(functionName,"*(%s= NFA::cross(\0",hname);
        break;
    case NODETYPE_DFA:
    	sprintf(functionName,"*(%s= DFA::cross(\0",hname);
        break;
    default:
        yyerror("The parameter for 'cross' function is invalid.");
        break;
    }


    code_size = strlen(functionName) + strlen(parameterlist->code) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),parameterlist->code,strlen(parameterlist->code));
    generated_code[strlen(functionName) + strlen(parameterlist->code)] = ')' ;
    generated_code[strlen(functionName) + strlen(parameterlist->code) + 1] = ')' ;

    return generated_code;

}



/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to reverse the parameter (one).
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *plus(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
        unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) reverse will only accept one parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'plus' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */
    unsigned char *hname = getHeapObjectName();
    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        //functionName = "*(hname = NFA::plus(\0";
    	sprintf(functionName,"*(%s= NFA::plus(\0",hname);
        break;
    case NODETYPE_DFA:
        //functionName = "*(hname = DFA::plus(\0";
    	sprintf(functionName,"*(%s= DFA::plus(\0",hname);
        break;
    case NODETYPE_REGEX:
        //functionName = "*(REGEX::plus(\0";
        sprintf(functionName,"*(REGEX::plus(\0");
        break;
    default:
        yyerror("The parameter for 'plus' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

	
	
    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);
	
	/*
	 * The pattern to be created:
	 * *(<functionName> ( <var_node.value_name> ))
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;

}




/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to reverse the parameter (one).
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *star(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) reverse will only accept one parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'star' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    unsigned char *hname = getHeapObjectName();
    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        //functionName = "*(hname=NFA::star(\0";
    	sprintf(functionName,"*(%s= NFA::star(\0",hname);
        break;
    case NODETYPE_DFA:
        //functionName = "*(hname=DFA::star(\0";
    	sprintf(functionName,"*(%s= DFA::star(\0",hname);
        break;
    case NODETYPE_REGEX:
        //functionName = "*(REGEX::star(\0";
        sprintf(functionName,"*(REGEX::star(\0");
        break;
    default:
        yyerror("The parameter for 'star' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

	
	
    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);
	
	/*
	 * The pattern to be created:
	 * *(<functionName> ( <var_node.value_name> ))
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;

}




/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to reverse the parameter (one).
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *reachable(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) reverse will only accept one parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'reachable' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    unsigned char *hname = getHeapObjectName();
    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        //functionName = "*(hname=NFA::reachable(\0";
    	sprintf(functionName,"*(%s= NFA::reachable(\0",hname);
        break;
    case NODETYPE_DFA:
        //functionName = "*(hname=DFA::reachable(\0";
    	sprintf(functionName,"*(%s= DFA::reachable(\0",hname);
        break;
    default:
        yyerror("The parameter for 'reachable' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

	
	
    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);
	
	/*
	 * The pattern to be created:
	 * *(<functionName> ( <var_node.value_name> ))
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;

}




/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to reverse the parameter (one).
* 			Only for types that are reversiable (ie. DFA and ...)
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *reduce(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) reverse will only accept one parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'reduce' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    unsigned char *hname = getHeapObjectName();
    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        //functionName = "*(hname=NFA::reduce(\0";
    	sprintf(functionName,"*(%s= NFA::reduce(\0",hname);
        break;
    case NODETYPE_DFA:
        //functionName = "*(hname=DFA::reduce(\0";
    	sprintf(functionName,"*(%s= DFA::reduce(\0",hname);
        break;
    default:
        yyerror("The parameter for 'reduce' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant. 
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

	
	
    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);
	
	/*
	 * The pattern to be created:
	 * *(<functionName> ( <var_node.value_name> ))
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;

}



unsigned char *complete(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) reverse will only accept one parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'complete' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    unsigned char *hname = getHeapObjectName();
    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        //functionName = "*(hname=NFA::reduce(\0";
    	sprintf(functionName,"*(%s= NFA::complete(\0",hname);
        break;
    case NODETYPE_DFA:
        //functionName = "*(hname=DFA::reduce(\0";
    	sprintf(functionName,"*(%s= DFA::complete(\0",hname);
        break;
    default:
        yyerror("The parameter for 'complete' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant.
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;



    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);

	/*
	 * The pattern to be created:
	 * *(<functionName> ( <var_node.value_name> ))
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;

}




unsigned char *nfatodfa(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) reverse will only accept one parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'nfatodfa' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    unsigned char *hname = getHeapObjectName();
    switch (parameterlist->simple_expression->type) {
    case NODETYPE_NFA:
        //functionName = "*(hname=NFA::reduce(\0";
    	sprintf(functionName,"*(%s= DFA::nfatodfa(\0",hname);
        break;
    default:
        yyerror("The parameter for 'nfatodfa' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant.
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;



    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);

	/*
	 * The pattern to be created:
	 * *(<functionName> ( <var_node.value_name> ))
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;

}


unsigned char *dfatonfa(int idSIndex, struct Parameter_List *parameterlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) reverse will only accept one parameter in the parameter list,
     * therefore we have to check there is only one single expression
     * in the parameter list. But, if the parameterlist had more than one
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (parameterlist->parameterlist != NULL || parameterlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'dfatonfa' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    unsigned char *hname = getHeapObjectName();
    switch (parameterlist->simple_expression->type) {
    case NODETYPE_DFA:
        //functionName = "*(hname=NFA::reduce(\0";
    	sprintf(functionName,"*(%s= NFA::dfatonfa(\0",hname);
        break;
    default:
        yyerror("The parameter for 'dfatonfa' function is invalid.");
        break;
    }
    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

	/*
	 * The parameter is a primitive data type, therefore is a constant.
	 * Thus we find the constant from symbol table in order to generate intermediate code.
	 */
    int variableIndex = parameterlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;



    symbol_node var_node = symbol_table[variableIndex];
    code_size = strlen(functionName) + strlen(var_node.value_name) + 4;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);

	/*
	 * The pattern to be created:
	 * *(<functionName> ( <var_node.value_name> ))
	 */
    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name) + 1] = ')' ;

    return generated_code;

}



/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			to read content of file
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *readfile(int idSIndex, struct Parameter_List *paramlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) "readfile" will only accept two parameter in the parameter list,
     * therefore we have to check there are only two simple-expression
     * in the parameter list. But, if the parameterlist had more/less than two
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (paramlist->parameterlist == NULL || paramlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'readfile' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    unsigned char *hname = getHeapObjectName();
    switch (paramlist->parameterlist->type) {
    case NODETYPE_NFA:
        //functionName = "*(hname=IO::read_nfa_file(\0";
        sprintf(functionName,"*(%s= IO::read_nfa_file(\0",hname);
        break;
    case NODETYPE_DFA:
        //functionName = "*(hname=IO::read_dfa_file(\0";
    	sprintf(functionName,"*(%s= IO::read_dfa_file(\0",hname);
        break;
    case NODETYPE_STRING:
        //functionName = "*(IO::read_string_file(\0";
    	sprintf(functionName,"*(NFA::reduce(\0");
        break;
    case NODETYPE_REGEX:
        //functionName = "*(IO::read_regex_file(\0";
    	sprintf(functionName,"*(NFA::reduce(\0");
        break;
    default:
        yyerror("The parameter for 'size' function is invalid.");
        break;
    }

    int variableIndex = paramlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

    symbol_node var_node = symbol_table[variableIndex];

    if (var_node.node_type != NODETYPE_STRING) {
        yyerror("The parameter for 'reafile' function is invalid.");
    }

    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

    code_size = strlen(functionName) + strlen(var_node.value_name) + 5;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);


    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name)+1] = ')' ;

    return generated_code;

}


unsigned char *parse(int idSIndex, struct Parameter_List *paramlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char functionName[100]; //Hold the function name should be called
    /*
     * (1) "readfile" will only accept two parameter in the parameter list,
     * therefore we have to check there are only two simple-expression
     * in the parameter list. But, if the parameterlist had more/less than two
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (paramlist->parameterlist == NULL || paramlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'parse' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    unsigned char *hname = getHeapObjectName();
    switch (paramlist->parameterlist->type) {
    case NODETYPE_NFA:
        //functionName = "*(hname=NFA::nfa_fromstring(\0";
    	sprintf(functionName,"*(%s= NFA::nfa_fromstring(\0",hname);
        break;
    case NODETYPE_DFA:
        //functionName = "*(hname=DFA::dfa_fromstring(\0";
    	sprintf(functionName,"*(%s= DFA::dfa_fromstring(\0",hname);
        break;
    case NODETYPE_REGEX:
        //functionName = "*(RE::re_fromstring(\0";
    	sprintf(functionName,"*(%s= RE::re_fromstring(\0",hname);
        break;
    default:
        yyerror("The parameter for 'parse' function is invalid.");
        break;
    }

    int variableIndex = paramlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

    symbol_node var_node = symbol_table[variableIndex];

    if (var_node.node_type != NODETYPE_STRING) {
        yyerror("The parameter for 'parse' function is invalid.");
    }

    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

    code_size = strlen(functionName) + strlen(var_node.value_name) + 5;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);


    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),var_node.value_name,strlen(var_node.value_name));
    generated_code[strlen(functionName) + strlen(var_node.value_name)] = ')' ;
    generated_code[strlen(functionName) + strlen(var_node.value_name)+1] = ')' ;

    return generated_code;

}



/*
* PURPOSE :	This is a predifined function that will generated intermediate code
* 			for writing parameter variable into a file
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *writefile(int idSIndex, struct Parameter_List *paramlist) {

    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char *functionName; //Hold the function name should be called
    /*
     * (1) "writefile" will only accept two parameter in the parameter list,
     * therefore we have to check there are only two simple-expression
     * in the parameter list. But, if the parameterlist had more/less than two
     * simple expression, the compilation process has to halt, and throw an error
     */
    if (paramlist->parameterlist == NULL || paramlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'writefile' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    switch (paramlist->parameterlist->type) {
    case NODETYPE_NFA:
        functionName = "IO::write_nfa_file(\0";
        break;
    case NODETYPE_DFA:
        functionName = "IO::write_dfa_file(\0";
        break;
    case NODETYPE_STRING:
        functionName = "IO::write_string_file(\0";
        break;
    case NODETYPE_REGEX:
        functionName = "IO::write_regex_file(\0";
        break;
    default:
        yyerror("The parameter for 'writefile' function is invalid.");
        break;
    }

    int variableIndex = paramlist->simple_expression->or_expression->unary_relation_expression->
                        relation_expression->add_expression1->term->factor->constant->sindex_constant;

    symbol_node var_node = symbol_table[variableIndex];

    if (var_node.node_type != NODETYPE_STRING) {
        yyerror("The parameter for 'writefile' function is invalid.");
    }

    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */

    code_size = strlen(functionName) + strlen(paramlist->code) + 5;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);


    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),paramlist->code,strlen(paramlist->code));
    generated_code[strlen(functionName) + strlen(paramlist->code)] = ')' ;


    return generated_code;
    //printf("--------------------------------------%s\n",generated_code);


}


/*
* PURPOSE :	This is a predifined function that will generate intermediate code
* 			for priting parameter(one) into standard output.
*
* INPUT:	symbol_table (global)
* 			idSIndex (parameter): index to a node in symbol table that represent
* 									predefined function ID
* 			parameterlist (parameter): partial syntax tree of parameters
* 
* RETURN :  string intermediate code generated for the function
*
* NOTES :	Be-careful about heap overflow.
*
* CHANGES:	None
*/
unsigned char *print(int idSIndex, struct Parameter_List *paramlist) {


    unsigned char *generated_code;
    int code_size = 0; //length of generated code
    unsigned char *functionName; //Hold the function name should be called
    /*
     * (1) "print" will only accept one parameter in the parameter list,
     * therefore we have to check there is only one simple-expression
     * in the parameter list. But, if the parameterlist had more/less than one
     * simple expression, the compilation process has to halt, and throw an error
     */

    if (paramlist->parameterlist != NULL || paramlist->simple_expression == NULL) {
        //throw syntax error.
        yyerror("No appropriate function's signiture for 'readfile' function");
    }


    /*
     * (2) We have to find out about the type of the simple-expression. And according
     * to the type, we have to generate appropriate C++/Intermediate code.
     * If the supplied type does not supported, compilation has to halt, and
     * appropriate error message be thrown.
     *
     * Supported Size function:
     * int, string, DFA, nfa, regex (maybe)
     */

    switch (paramlist->simple_expression->type) {
    case NODETYPE_NFA:
        functionName = "IO::print_nfa(\0";
        break;
    case NODETYPE_DFA:
        functionName = "IO::print_dfa(\0";
        break;
    case NODETYPE_STRING:
        functionName = "IO::print_string(\0";
        break;
    case NODETYPE_REGEX:
        functionName = "IO::print_regex(\0";
        break;
    case NODETYPE_INTEGER:
        functionName = "IO::print_int(\0";
        break;
    default:
        yyerror("The parameter for 'print' function is invalid.");
        break;
    }


    struct Constant *cons =  paramlist->simple_expression->or_expression->unary_relation_expression->
                                         relation_expression->add_expression1->term->factor->constant;

    /*
     *(3) The generated code has to be returned, and will be the end of this function.
    */ /*

unsigned char *parameters;

if(cons->function == NULL){
int variableIndex = cons->sindex_constant;
symbol_node var_node = symbol_table[variableIndex];
parameters = var_node.value_name;
}else{
parameters = cons->function->code;
}

struct Constant *cons =  paramlist->simple_expression->or_expression->unary_relation_expression->
					relation_expression->add_expression1->term->factor->constant;

/*
*(3) The generated code has to be returned, and will be the end of this function.
*/

    unsigned char *parameters;

    if (cons->function == NULL) {
        int variableIndex = cons->sindex_constant;
        symbol_node var_node = symbol_table[variableIndex];
        //parameters = var_node.value_name;
        parameters = paramlist->simple_expression->code;
    } else {
        parameters = cons->function->code;
    }

    code_size = strlen(functionName) + strlen(parameters) + 5;
    generated_code = (unsigned char *)malloc(sizeof(unsigned char) * code_size);
    memset(generated_code,'\0',code_size);


    memcpy(generated_code,functionName,strlen(functionName));
    memcpy(&(generated_code[strlen(functionName)]),parameters,strlen(parameters));
    generated_code[strlen(functionName) + strlen(parameters)] = ')' ;

    return generated_code;

    //printf("--------------------------------------%s\n",generated_code);

}
