/*
*    Language Investigator and Stractural Analyzer (LISA) -
*    A descriptive programming language
 * LICENCE LGPL
 * FILE:					symboltable.c
 *
 * PURPOSE:				Create and maintain the symbol table.
 *
 * CREATED AT:			November 6, 2011
 *
 * AUTHOR: 		Seyed Pooria Madani Kochak (s.p.madani@gmail.com)
 *
 * Project Manager:	Dr. Cezar Campeanu (ccampeanu@upei.ca)
 *
 * Modified by:
 * Brandon Stewart     - JSON symbol loading, namespace support, and restructuring, details in
 *                     "lisa-changelist.txt".                      June/July 2015

*/

#include "symboltable.h"
#include <stdio.h>
#include <stdlib.h>
#ifdef __unix__
#include <dirent.h>
#elif __WIN32
#include <windows.h>
#endif


symbol_node *symbol_table;
int number_of_nodes = 0;
int capacity_of_symbol_table = INIT_CAPACITY;

const char* DATA_SYMBOLS_DIR   = "../src/data/symbols/";


/*
 * PURPOSE :	 This function will dynamically allocate the symbol table by the provided size
 *	    	(INIT_CAPACITY). It will load all the keywords, predefined symbols, and functions
 *	    	into the table.
 *
 * INPUT:	symbol_table (global)
 *
 * RETURN :  	None
 *
 * NOTES :	Be-careful about heap overflow.
 *
 * CHANGES:
*				Brandon Stewart: symbols now loaded in via json.
 */
void init_symbol_table(void) {

	symbol_table = (symbol_node *) malloc(sizeof(symbol_node) * INIT_CAPACITY);

#ifdef __unix__

	DIR* d;
	struct dirent* dir;

	d = opendir(DATA_SYMBOLS_DIR);
	if (d)
	{
		while ((dir = readdir(d)))
    	{
			//make sure is just a file
			if (dir->d_type == DT_REG)
			{
				int size = strlen(DATA_SYMBOLS_DIR) + strlen(dir->d_name) + 1; //+1 for null
				char filepath[size];

				snprintf(filepath, size, "%s%s", DATA_SYMBOLS_DIR, dir->d_name);

				load_json_with_namespace(filepath);
			}
    	}

		closedir(d);
	}

#elif __WIN32

	WIN32_FIND_DATA file_data;
	HANDLE file_handle;

	file_handle = FindFirstFile(DATA_SYMBOLS_DIR, &file_data);

	if (file_handle == INVALID_HANDLE_VALUE)
	{
		yyerror("Error reading symbol files (src/data/symbols/)");
	}

	do
	{
		//is supposed to be bitwise and, not "&&"
		if (file_data.dwFileAttributes & FILE_ATTRIBUTE_NORMAL)
		{
			int size = strlen(DATA_SYMBOLS_DIR) + strlen(file_data.cFileName) + 1; //+1 for null
			char filepath[size];

			snprintf(filepath, size, "%s%s", DATA_SYMBOLS_DIR, file_data.cFileName);

			load_json_with_namespace(filepath);
		}
	}
	while (FindNextFile(file_handle, &file_data));

#endif


}


void load_json_with_namespace(char* filepath)
{

	symbol_node *node;
    char* code = file_contents(filepath); //"file_contents" from filereader.h
    const nx_json* json = nx_json_parse(code, 0);

    if (json)
    {
		const nx_json* jsonNode;

		int namespaceID = -1;

//namespace, special case
		const nx_json* jsonNamespace = nx_json_get(json, "namespace");
		node = (symbol_node *)  malloc(sizeof(symbol_node));
		if (jsonNamespace && nx_json_get(jsonNamespace, "value_name")->text_value)
		{
			namespaceID = number_of_nodes;
			node->namespace_type = namespaceID;
			node->value_name = strdup(nx_json_get(jsonNamespace, "value_name")->text_value);
			node->value = strdup(nx_json_get(jsonNamespace, "value")->text_value);
			node->token_type = nx_json_get(jsonNamespace, "token_type")->int_value;
			node->node_type = nx_json_get(jsonNamespace, "node_type")->int_value;
			node->param_data.num_params = 0;
			node->param_data.param_types = NULL;
			node->param_data.returns_pointer = 0;
			jsonNode = json->child->next;

			add_symbol_node(node);
		}
		else
		{
			jsonNode = json->child;
		}
//everything else that isn't a namespace token
		while(jsonNode)
		{
			load_symbols_from_json(
				namespaceID,
				nx_json_get(jsonNode, "token_type")->int_value,
				jsonNode->child->next
			);

			jsonNode = jsonNode->next;
		}


	}
	nx_json_free(json);
	free(code);
	free(node);

}

void load_symbols_from_json(int namespace, int tokenType, const nx_json* jsonArray)
{

	symbol_node *node = malloc(sizeof(symbol_node));
	int i, j;
	for (i = 0; jsonArray && (i < jsonArray->length); i++)
	{

		const nx_json* item = nx_json_item(jsonArray, i);

		node->namespace_type = namespace;
		node->token_type 	 = tokenType;
		node->value_name 	 = strdup(nx_json_get(item, "value_name")->text_value);

		//if "value" exists, duplicate the result, or set to NULL
		node->value			 =  nx_json_get(item, "value")->text_value ?
								strdup(nx_json_get(item, "value")->text_value) :
								NULL;
		node->node_type  	 = nx_json_get(item, "node_type")->int_value;

		//get parameter data:
		const nx_json* jsonParameters = nx_json_get(item, "parameters");
		//if there are parameters, get the amount.  Otherwise, set to zero.
		node->param_data.num_params = jsonParameters ? jsonParameters->length : 0;
		node->param_data.param_types = node->param_data.num_params ?
						malloc(sizeof(enum symbol_node_type) * node->param_data.num_params) :
						NULL;
		node->param_data.returns_pointer = nx_json_get(item, "returns_pointer")->int_value;
		//printf("NODE: %d %s, %d\n", namespace, node->value_name, node->param_data.returns_pointer);
		for (j = 0; (j < node->param_data.num_params); j++)
		{
			const nx_json* param = nx_json_item(jsonParameters, j);
			node->param_data.param_types[j] = param->int_value;
		}

		add_symbol_node(node);


	}

	free(node);

}

//frees the memory used by symbol table
void delete_symbol_table(void)
{

	symbol_node* temp_node;
	int i;
	for (i = 0; i < number_of_nodes; i++)
	{
		temp_node = &symbol_table[i];
		//printf("NODE %d: %s %s \n", i, temp_node->value_name, temp_node->value);

		free(temp_node->value_name);
		//free(temp_node->value);  //sometimes this doesn't get set.  This should be fixed to
								   //be initialized to null.
		if (temp_node->param_data.num_params > 0)
			free(temp_node->param_data.param_types);


	}
	free (symbol_table);
	symbol_table = NULL;

	number_of_nodes = 0;
	capacity_of_symbol_table = INIT_CAPACITY; //back to default

}



/*
 * PURPOSE :	This function will add a new node into the Symbol Table. It will expand the size of
 *		the symbol table if it is necessary. Search to make sure the node to be added does not
 *		exist in the symbol table. If it does exist, function will return the existing index to the
 *		symbol table for the node. Otherwise, it will create new index and will setup the node into
 *		symbol table.
 *
 * INPUT:	symbol_node *node(parameter)	The node that needs to be added to the symbol table.
 *		symbol_table (global)		The symbol table, inorder to add the new item into it.
 *
 * RETURN :  	int	Index to the symbol table that the node has been added to.
 *
 * NOTES :	Be-careful about heap overflow.
 *
 * CHANGES:	None
 */
int add_symbol_node(symbol_node *node) {

	//Check to see if the symbol table needs to be expanded
	if (number_of_nodes >= capacity_of_symbol_table) {
		expand_symbol_table(); //increase the size of the symbol table
	}

	/*
	 * check to see if the node exists or not
	 */
	int i;
	if ((i = find_symbol_node(node)) == -1) {
		node->array_capacity = 0;
		symbol_table[number_of_nodes] = *node;
		return number_of_nodes++;
	} else {
		return i;
	}
}

/*
 * PURPOSE :	This function will search for the input node on the symbol table. If the node_type
 *		and value_name of the given node matched any node inside the symbol table, function will return
 *		index number to that particular node. Otherwise it will return -1.
 *
 * INPUT:	symbol_node *node(parameter)	The node that needs to be finded in the symbol table.
 *		symbol_table (global)		The symbol table, inorder to search for the node.
 *		number_of_nodes			Number of the nodes that are in the symbol table.
 *
 * RETURN :  	int	Index to the symbol table that the node has been found to. Or
 *			-1 if the node couldn't be found.
 *
 * NOTES :	Be-careful about heap overflow.
 *		The search method, linearly will scan the whole table to find
 *		the node. There is no big performance lost, but if this gets
 *		reimplemented by a Hashfunction, a better performance we might
 *		get. I suggest for future upgrades, we look at implementing
 *		the symbol table using a hash function.
 *
 * CHANGES:
 *
 * Brandon Stewart - Besides searching by value name, it now checks by namespace as well.
 */
int find_symbol_node(symbol_node *node) {
//	printf("FIND SYMBOL NODE &&&&&& %s (node-type =%d) (n_params=%d)\n", node->value_name, node->node_type, node->param_data.num_params);
	int returnval = -1;
	int i;
	int value_name1, value_name2;
	for (i = 0; i < number_of_nodes; i++) {

		symbol_node tmp_node = symbol_table[i];
		/*
		 * Most of the input types are ether String, Integer, Float
		 * float points are not supported yet. Thus, all the value_names
		 * are stored in terms of Integer or String(char *)
		 */
		switch (tmp_node.token_type) {
		case TOKENTYPE_NUMBER:
			value_name1 = *((int *) tmp_node.value_name);
			value_name2 = *((int *) node->value_name);
			if (value_name1 == value_name2)
				returnval = i;
			break;

		case TOKENTYPE_PREDEFINED_FUNCTION:

			if (node->token_type != TOKENTYPE_PREDEFINED_FUNCTION &&
				strcmp(node->value_name,tmp_node.value_name) == 0)
				{
					return i;
				}

			if ((strcmp(tmp_node.value_name, node->value_name) == 0) &&
			   (tmp_node.namespace_type == node->namespace_type) &&
			   (tmp_node.param_data.num_params == node->param_data.num_params))
			{
				//compare parameters
				int equal = 1;
				int j;
				for (j = 0; equal && j < tmp_node.param_data.num_params; j++)
				{

					if (tmp_node.param_data.param_types[j] != node->param_data.param_types[j])
						equal = 0;
				}
				if (equal)
				{
					returnval = i;
				}


			}
			break;

		case TOKENTYPE_KEYWORD:
			if ((node->token_type == TOKENTYPE_KEYWORD) &&
				(strcmp(node->value_name,tmp_node.value_name) == 0) &&
				(node->namespace_type == tmp_node.namespace_type))
			{
				return i;
			}
			else if ((node->token_type == TOKENTYPE_KEYWORD) &&
				(strcmp(node->value_name,tmp_node.value_name) == 0) &&
				(node->namespace_type != tmp_node.namespace_type))
			{
			/*	printf("Same except NS----------%s---------------%s-------------------------\n",
				node->value_name,tmp_node.value_name
				);
				//return -1;
				*/
			}
			else
			{
				if (strcmp(tmp_node.value_name, node->value_name) == 0)
				{
						returnval = i;
				}
			}

			break;
		default:
			if (strcmp(tmp_node.value_name, node->value_name) == 0)
			{

					returnval = i;
			}
			break;
		}
		if (returnval > -1)
			break;
	}
	//printf("CHECK %d\n", returnval);
	return returnval;
}

/*
 * PURPOSE :	This method will expand the size of the symbol table by the constant give (INPUT).
 *		This makes the symbol table to function as dynamic array.
 *
 * INPUT:	symbol_table (global)		The symbol table, inorder to expand its size.
 *		INCREASE_BY (global)		Constant that will tell how much add to the existing
 capacity.
 *
 * RETURN :  	None
 *
 * NOTES :	Be careful about heap overflow.
 *
 * CHANGES:	None
 */
void expand_symbol_table(void) {

	capacity_of_symbol_table += INCREASE_BY; //Increase the capacity by the constant

	symbol_node *tmp_symbol_table = (symbol_node *) realloc(symbol_table,
			sizeof(symbol_node) * capacity_of_symbol_table);
	if (tmp_symbol_table == NULL) {
		free(symbol_table);
		fprintf(stdout, "** Error reallocating memory for the Symbol Table.\n");
	}
	symbol_table = tmp_symbol_table;

	//This line is for debugging purposes.
	printf("Symbol table has been expanded to %d\n", capacity_of_symbol_table);

}

/*
 * PURPOSE :	It will print the symbol table on stdout.
 *
 * INPUT:	symbol_table (global)		The symbol table, inorder to print it.
 *		number_of_nodes			Number of the nodes that are in the symbol table.
 *
 * RETURN :  	None
 *
 * NOTES :	Be-careful about heap overflow.
 *
 * CHANGES:	None
 */
void print_symbol_table(void) {
	int i;
	for (i = 0; i < number_of_nodes; i++) {
		symbol_node node = symbol_table[i];

		switch (node.token_type) {
		case TOKENTYPE_NUMBER:
			printf("%d \t %s(%d)(TOKENTYPE_NUMBER)", i,
					(char *) node.value_name, *((int *) node.value));
			break;

		case TOKENTYPE_ID:
			printf("%d \t %s(TOKENTYPE_ID)", i, (char *) node.value_name);
			break;

		case TOKENTYPE_KEYWORD:
			printf("%d \t %s(TOKENTYPE_KEYWORD)", i, (char *) node.value_name);
			break;

		case TOKENTYPE_SYMBOL:
			printf("%d \t %s(TOKENTYPE_SYMBOL)", i, (char *) node.value_name);
			break;

		case TOKENTYPE_STRINGLITERALS:
			printf("%d \t %s(%s)(TOKENTYPE_STRINGLITERALS)", i,
					(char *) node.value_name, (char *) node.value);
			break;

		case TOKENTYPE_PREDEFINED_FUNCTION:
			printf("%d \t %s(TOKENTYPE_PREDEFINED_FUNCTION)", i,
					(char *) node.value_name);
			break;

		case TOKENTYPE_NAMESPACE:
			printf("%d \t %s(TOKENTYPE_NAMESPACE)", i,
					(char *) node.value_name);
			break;

		}

		if (node.namespace_type >= 0 && node.namespace_type < number_of_nodes)
		{
			symbol_table[node.namespace_type].value_name;
			printf("(NAMESPACE_%s)", symbol_table[node.namespace_type].value_name);
		}
		else
		{
			printf("(NAMESPACE_null)");
		}


		printf("(returns_ptr:%d)", node.param_data.returns_pointer);

		printf("(arraycapacity=%d)", node.array_capacity);

		switch (node.node_type) {
		case NODETYPE_DFA:
			printf("(NODETYPE_DFA)\n");
			break;

		case NODETYPE_NFA:
			printf("(NODETYPE_NFA)\n");
			break;

		case NODETYPE_REGEX:
			printf("(NODETYPE_REGEX)\n");
			break;

		case NODETYPE_STRING:
			printf("(NODETYPE_STRING)\n");
			break;

		case NODETYPE_INTEGER:
			printf("(NODETYPE_INTEGER)\n");
			break;

		case NODETYPE_BOOLEAN:
			printf("(NODETYPE_BOOLEAN)\n");
			break;
		case NODETYPE_FMPOINTER:
			printf("(NODETYPE_FMPOINTER)\n");
			break;

		default:
			printf("(NULL-%d)\n", node.node_type);
			break;
		}



	}

	printf("\n*************************\nSummary of the Symbol Table\n*************************\n");
	printf("\t Current Capacity (Expandable):%d\n", capacity_of_symbol_table);
	printf("\t Number of symbols:%d\n", number_of_nodes);
	printf("*************************\n");
}

/*
 * PURPOSE :	This is a specilized function that add Identifier node into symbol table
 * 			by only asking of the node name.
 *
 * INPUT:	symbol_table (global)
 * 			str (parameter): input string that will be value_name of the node
 *
 * RETURN :  INT: index to symbol table of node that asked to be added.
 *
 * NOTES :	Be-careful about heap overflow.
 *
 * CHANGES:	None
 */
int add_id_symbol_node(char *str) {

	symbol_node *node = (symbol_node *) malloc(sizeof(symbol_node));
	node->value_name = str;
	node->token_type = TOKENTYPE_ID;
	node->node_type = NODETYPE_NULL;
	node->param_data.returns_pointer = 0;
	node->param_data.num_params = 0;
	node->param_data.param_types = NULL;


	return add_symbol_node(node);

}

/*
 * PURPOSE :	This is a specilized function that add String literal node into symbol table
 * 			by only asking of the node name.
 *
 * INPUT:	symbol_table (global)
 * 			str (parameter): input string that will be value_name of the node
 *
 * RETURN :  INT: index to symbol table of node that asked to be added.
 *
 * NOTES :	Be-careful about heap overflow.
 *
 * CHANGES:	None
 */
int add_stringliterals_symbol_node(char *str) {

	symbol_node *node = (symbol_node *) malloc(sizeof(symbol_node));

	char *intvalue = (char *) malloc(sizeof(unsigned char) * 9);
	memset(intvalue, '\0', sizeof(unsigned char) * 9);
	sprintf(intvalue, "%d ", number_of_nodes);

	int size = strlen("str") + strlen(intvalue) + 1;
	char *vname = (char*) malloc(sizeof(char) * size);
	memset(vname, '\0', size);

	memcpy(vname, "str", strlen("str"));
	memcpy(&(vname[strlen("str")]), intvalue, strlen(intvalue));

	node->value_name = vname; //str + number_of_nodes
	node->value = str;
	node->token_type = TOKENTYPE_STRINGLITERALS;
	node->node_type = NODETYPE_STRING;
	node->namespace_type = -1; //global
	node->param_data.returns_pointer = 0;
	node->param_data.num_params = 0;
	node->param_data.param_types = NULL;

	return add_symbol_node(node);

}

int add_character_symbol_node(char *str) {

	symbol_node *node = (symbol_node *) malloc(sizeof(symbol_node));

	node->value_name = str; //str + number_of_nodes
	node->value = str;
	node->token_type = TOKENTYPE_STRINGLITERALS;
	node->node_type = NODETYPE_STRING;
	node->param_data.returns_pointer = 0;
	node->param_data.num_params = 0;
	node->param_data.param_types = NULL;

	return add_symbol_node(node);

}

/*
 * PURPOSE :	This function will set type of variableIndex node to typeIndex node
 *
 * INPUT:	symbol_table (global)
 * 			variableIndex (parameter): index to a node in symbol table for which the
 * 										type needs to be set.
 * 			typeIndex (parameter): index to a node in symbol table which the type will
 * 									be taken from accordingly
 *
 * RETURN :  None
 *
 * NOTES :	Be-careful about heap overflow.
 *
 * CHANGES:	None
 */

void set_variable_type(int variableIndex, int typeIndex) {

	symbol_node *variableNode = &(symbol_table[variableIndex]);
	symbol_node typeNode = symbol_table[typeIndex];

	//if its in the symbol table, it has to be a valid index (>= zero)
	if (typeNode.namespace_type >= 0)
	{	//we need to check the stack
		if (StackIsEmpty(namespaceStack))
			yyerror("unknown type");

		symbol_node tmp_node = symbol_table[namespaceStack->data];
		if (tmp_node.namespace_type != typeNode.namespace_type)
		{
			//check if such a type exists under the namespace
			typeNode.namespace_type = tmp_node.namespace_type;
			//print_symbol_table();
			int a = find_symbol_node(&typeNode);
			if (a == -1)
				yyerror("unknown type in namespace");

			typeNode = symbol_table[a];
		}

	}

	variableNode->namespace_type = typeNode.namespace_type;
	variableNode->value = typeNode.value;
	variableNode->node_type = typeNode.node_type;
	variableNode->array_capacity = 0; //When array capacity is zero, it means the variable is not array

}

void namespace_set_variable_type(int namespaceIndex, int variableIndex, int typeIndex) {

	symbol_node *variableNode = &(symbol_table[variableIndex]);
	symbol_node typeNode = symbol_table[typeIndex];
	symbol_node namespaceNode = symbol_table[namespaceIndex];


	if (namespaceNode.token_type != TOKENTYPE_NAMESPACE)
	{
		int size = strlen("Unknown namespace \"\"") + strlen(namespaceNode.value_name) + 1; //+1 for null
		char errorMsg[size];
		snprintf(errorMsg, size, "Unknown namespace \"%s\"", namespaceNode.value_name);
		yyerror(errorMsg);

	}

	if (typeNode.namespace_type != namespaceNode.namespace_type)
	{
		//check if such a type exists under the namespace
		typeNode.namespace_type = namespaceNode.namespace_type;

		int a = find_symbol_node(&typeNode);
		if (a == -1)
			yyerror("unknown type in namespace");

		typeNode = symbol_table[a];

	}


	variableNode->value = typeNode.value;
	variableNode->node_type = typeNode.node_type;
	variableNode->namespace_type = namespaceNode.namespace_type;
	variableNode->array_capacity = 0; //When array capacity is zero, it means the variable is not array

}

void set_array_type(int variableIndex, int typeIndex, int capacityIndex) {

	symbol_node *variableNode = &(symbol_table[variableIndex]);
	symbol_node capacityNode = symbol_table[capacityIndex];
	symbol_node typeNode = symbol_table[typeIndex];

	//if its in the symbol table, it has to be a valid index (>= zero)
	if (typeNode.namespace_type >= 0)
	{	//we need to check the stack
		if (StackIsEmpty(namespaceStack))
			yyerror("unknown type");

		symbol_node tmp_node = symbol_table[namespaceStack->data];
		if (tmp_node.namespace_type != typeNode.namespace_type)
		{
			//check if such a type exists under the namespace
			typeNode.namespace_type = tmp_node.namespace_type;

			int a = find_symbol_node(&typeNode);
			if (a == -1)
				yyerror("unknown type in namespace");

			typeNode = symbol_table[a];
		}

	}
	variableNode->value = typeNode.value;
	variableNode->node_type = typeNode.node_type;
	variableNode->namespace_type = typeNode.namespace_type;
	variableNode->array_capacity = *((int *) (capacityNode.value)); //When array capacity is zero, it means the variable is not array

}

void namespace_set_array_type(int namespaceIndex,int variableIndex, int typeIndex, int capacityIndex) {

	symbol_node *variableNode = &(symbol_table[variableIndex]);
	symbol_node capacityNode = symbol_table[capacityIndex];
	symbol_node typeNode = symbol_table[typeIndex];
	symbol_node namespaceNode = symbol_table[namespaceIndex];


	if (namespaceNode.token_type != TOKENTYPE_NAMESPACE)
	{
		int size = strlen("Unknown namespace \"\"") + strlen(namespaceNode.value_name) + 1; //+1 for null
		char errorMsg[size];
		snprintf(errorMsg, size, "Unknown namespace \"%s\"", namespaceNode.value_name);
		yyerror(errorMsg);
	}


	if (typeNode.namespace_type != namespaceNode.namespace_type)
	{
		//check if such a type exists under the namespace
		typeNode.namespace_type = namespaceNode.namespace_type;

		int a = find_symbol_node(&typeNode);
		if (a == -1)
			yyerror("unknown type array in namespace");

		typeNode = symbol_table[a];

	}

	variableNode->value = typeNode.value;
	variableNode->node_type = typeNode.node_type;
	variableNode->namespace_type = namespaceNode.namespace_type;
	variableNode->array_capacity = *((int *) (capacityNode.value)); //When array capacity is zero, it means the variable is not array

}


/*
 * PURPOSE :	This function will add integer node into symbol table
 *
 * INPUT:	symbol_table (global)
 * 			i (parameter): integer to be added to symboltable

 *
 * RETURN :  None
 *
 * NOTES :	Be-careful about heap overflow.
 *
 * CHANGES:
 */
int add_number_symbol_node(char *c, int i) {
	symbol_node *node = (symbol_node *) malloc(sizeof(symbol_node));
	int *num = (int *) malloc(sizeof(int));
	*num = i;
	node->value_name = c;
	node->value = num; //
	node->node_type = NODETYPE_INTEGER;
	node->token_type = TOKENTYPE_NUMBER;
	node->namespace_type = -1; //global
	node->param_data.returns_pointer = 0;
	node->param_data.num_params = 0;
	node->param_data.param_types = NULL;

	return add_symbol_node(node);
}

int add_finitemachine_pointer_symbol_node(char *name) {
	symbol_node *node = (symbol_node *) malloc(sizeof(symbol_node));

	node->value_name = name; //str + number_of_nodes
	node->value = name; //c++ code equivalent
	node->token_type = TOKENTYPE_ID;
	node->node_type = NODETYPE_FMPOINTER;
	node->param_data.returns_pointer = 0;
	node->param_data.num_params = 0;
	node->param_data.param_types = NULL;

	return add_symbol_node(node);
}



int count_symbols_same_declaration(symbol_node *node)
{
	int count = 0;
	int i;
	for (i = 0; i < number_of_nodes; i++) {
		symbol_node tmp_node = symbol_table[i];

		if (node->namespace_type != tmp_node.namespace_type &&
			strcmp(node->value_name, tmp_node.value_name) == 0 &&
			node->param_data.num_params == tmp_node.param_data.num_params)
		{
			//check parameters, if they are the same, increase count
			int equal = 1;
			int j;
			for (j = 0; equal && j < tmp_node.param_data.num_params; j++)
			{
				if (tmp_node.param_data.param_types[j] != node->param_data.param_types[j])
					equal = 0;

			}

			if (equal)
			{
				count++;
			}
		}

	}

	return count;

}
